"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transform = void 0;
/**
 * This plugin works the same way as
 * https://www.npmjs.com/package/babel-plugin-transform-react-jsx-source
 * which inject __source={{ fileName, lineNumber }} into
 * every React Element so React can debug
 * Ref: https://github.com/facebook/react/blob/b87aabdfe1b7461e7331abb3601d9e6bb27544bc/packages/shared/describeComponentFrame.js#L35
 */
const TYPESCRIPT = require("typescript");
function nodeVisitor(ts, ctx, sf) {
    let sourceJsxAttr;
    const visitor = (node) => {
        if (ts.isJsxOpeningElement(node) || ts.isJsxSelfClosingElement(node)) {
            // Create fileName attr
            const fileNameAttr = ctx.factory.createPropertyAssignment('fileName', ctx.factory.createStringLiteral(sf.fileName));
            // Create lineNumber attr
            const lineNumberAttr = ctx.factory.createPropertyAssignment('lineNumber', ctx.factory.createNumericLiteral(ts.getLineAndCharacterOfPosition(sf, node.pos).line + 1));
            // Create __source={{fileName, lineNumber}} JSX Attribute
            sourceJsxAttr = ctx.factory.createJsxAttribute(ctx.factory.createIdentifier('__source'), ctx.factory.createJsxExpression(undefined, ctx.factory.createObjectLiteralExpression([
                fileNameAttr,
                lineNumberAttr,
            ])));
        }
        else if (ts.isJsxAttributes(node) && sourceJsxAttr) {
            const attrs = [...node.properties, sourceJsxAttr];
            sourceJsxAttr = undefined;
            return ctx.factory.updateJsxAttributes(node, attrs);
        }
        return ts.visitEachChild(node, visitor, ctx);
    };
    return visitor;
}
function transform(ts = TYPESCRIPT) {
    return (ctx) => (sf) => ts.visitNode(sf, nodeVisitor(ts, ctx, sf));
}
exports.transform = transform;
